// CircularDualNode.h

#pragma once // for visual studio.
#ifndef CIRCULARDUALNODE_H // for other compilers.
#define CIRCULARDUALNODE_H

/* recommended. */
#include "DualNode.h"
/*/
// use this if something wrong with code before.
#ifndef DUALNODE_H
#include "DualNode.h"
#endif // end for #ifndef DUALNODE_H.
//*/

template<typename T>
//  ()    T  2     CircularDualNode<T> *.
//    .
struct CircularDualNode : protected DualNode<T>
{
private:
	typedef DualNode<T> Dual;
	typedef CircularDualNode<T> Circular; // make code below smaller.

public:
	//  .
	virtual T const &GetData() const { return Dual::GetData(); }
	//  .
	virtual void SetData(T const &data) { Node<T>::SetData(data); }

	virtual Circular *const GetNext() const { return static_cast<Circular *>(Node<T>::GetNext()); }
	//      next,   
	//    next'    (this).
	virtual void SetNext(Circular *const &next) { if(next != nullptr) Dual::SetNext(next); }
	//virtual void SetNext(Circular &next) { Circular::SetNext(&next); }

	virtual Circular *const GetPrev() const { return static_cast<Circular *>(Dual::GetPrev()); }
	//      prev,   
	//    prev'    (this).
	virtual void SetPrev(Circular *const &prev) { if(prev != nullptr) Dual::SetPrev(prev); }
	//virtual void SetPrev(Circular &prev) { Circular::SetPrev(&prev); }

	/* use this code if compiler support C++11 or higher */

	CircularDualNode() : Dual(T(), this, this) { }
	CircularDualNode(T const &data) : Dual(data, this, this) { }
	CircularDualNode(T const &data, Circular *const &next)
		: Circular(data) { Circular::SetNext(next); }
	//CircularDualNode(T const &data, Circular &next) : Circular(data, &next) { }
	CircularDualNode(T const &data, Circular *const &next, Circular *const &prev)
		: Circular(data, next) { Circular::SetPrev(prev); }
	//CircularDualNode(T const &data, Circular &next, Circular &prev) : Circular(data, &next, &prev) { }
	CircularDualNode(Circular const &lvalue) : Dual(lvalue.Circular::GetData(), this, this) { }

	CircularDualNode(T &&data) : Dual(std::move<T &>(data), this, this) { }
	CircularDualNode(T &&data, Circular *const &next)
		: Circular(std::move<T &>(data)) { Circular::SetNext(next); }
	CircularDualNode(T &&data, Circular *const &next, Circular *const &prev)
		: Circular(std::move<T &>(data), next) { Circular::SetPrev(prev); }
	CircularDualNode(Circular &&rvalue) : Dual(std::move<Circular &>(rvalue))
	{
		rvalue.Node<T>::SetNext(&rvalue);
		rvalue.Dual::SetPrev(&rvalue);
	}
	/*/
	// use code below if code before generate error.

	CircularDualNode() { Dual(T(), this, this); }
	CircularDualNode(T const &data) { Dual(data, this, this); }
	CircularDualNode(T const &data, Circular *const next)
	{
		Dual(data, this, this);
		Circular::SetNext(next);
	}
	CircularDualNode(T const &data, Circular &next)
	{
		Dual(data, this, this);
		Circular::SetNext(&next);
	}
	CircularDualNode(T const &data, Circular *const next, Circular *const prev)
	{
		Dual(data, this, this);
		Circular::SetNext(next);
		Circular::SetPrev(prev);
	}
	CircularDualNode(T const &data, Circular &next, Circular &prev)
	{
		Dual(data, this, this);
		Circular::SetNext(&next);
		Circular::SetPrev(&prev);
	}
	CircularDualNode(Circular const &lvalue) { Dual(lvalue.Circular::GetData(), this, this); }
	//*/

	virtual ~CircularDualNode() { } // empty destructor is optional.

	// Copy assignment operator.
	virtual Circular const &operator=(Circular const &lvalue)
	{
		Circular::SetData(lvalue.Circular::GetData());
		Node<T>::SetNext(this);
		Dual::SetPrev(this);
		return *this;
	}
	// Move assignment operator.
	virtual Circular const &operator=(Circular &&rvalue)
	{
		Dual::operator=(std::move<Circular &>(rvalue));
		rvalue.Node<T>::SetNext(&rvalue);
		rvalue.Dual::SetPrev(&rvalue);
		return *this;
	}

protected:
	//      next,   
	//    next'    (this).
	virtual void SetNext(Dual *const &next) { SetNext(static_cast<Circular *>(next)); }
	//      prev,   
	//    prev'    (this).
	virtual void SetPrev(Dual *const &prev) { SetPrev(static_cast<Circular *>(prev)); }
};
#endif // end for #ifndef CIRCULARDUALNODE_H.
